/* ===========================================================
# sphere-orderstatus-xml-import - v0.1.0
# ==============================================================
# Copyright (c) 2013,2014 Sven Mueller
# Licensed under the MIT license.
*/
var ExtendedLogger, OrderStatusImport, ProjectCredentialsConfig, Promise, SftpHelper, argv, fs, importFn, logOptions, logger, package_json, path, readJsonFromPath, tmp, _, _ref;

path = require('path');

_ = require('underscore');

Promise = require('bluebird');

fs = Promise.promisifyAll(require('fs'));

tmp = Promise.promisifyAll(require('tmp'));

_ref = require('sphere-node-utils'), ExtendedLogger = _ref.ExtendedLogger, ProjectCredentialsConfig = _ref.ProjectCredentialsConfig;

package_json = require('../package.json');

OrderStatusImport = require('./orderstatusimport');

SftpHelper = require('./sftp');

argv = require('optimist').usage('Usage: $0 --projectKey [key] --clientId [id] --clientSecret [secret] --file [file] --logDir [dir] --logLevel [level]').describe('projectKey', 'your SPHERE.IO project-key').describe('clientId', 'your OAuth client id for the SPHERE.IO API').describe('clientSecret', 'your OAuth client secret for the SPHERE.IO API').describe('sphereHost', 'SPHERE.IO API host to connecto to').describe('file', 'XML file containing inventory information to import').describe('sftpCredentials', 'the path to a JSON file where to read the credentials from').describe('sftpHost', 'the SFTP host (overwrite value in sftpCredentials JSON, if given)').describe('sftpUsername', 'the SFTP username (overwrite value in sftpCredentials JSON, if given)').describe('sftpPassword', 'the SFTP password (overwrite value in sftpCredentials JSON, if given)').describe('sftpSource', 'path in the SFTP server from where to read the files').describe('sftpTarget', 'path in the SFTP server to where to move the worked files').describe('sftpFileRegex', 'a RegEx to filter files when downloading them').describe('sftpMaxFilesToProcess', 'how many files need to be processed, if more then one is found').describe('sftpContinueOnProblems', 'ignore errors when processing a file and continue with the next one').describe('sftpFileWithTimestamp', 'whether the processed file should be renamed by appending a timestamp').describe('logLevel', 'log level for file logging').describe('logDir', 'directory to store logs').describe('logSilent', 'use console to print messages').describe('timeout', 'Set timeout for requests')["default"]('logLevel', 'info')["default"]('logDir', '.')["default"]('logSilent', false)["default"]('timeout', 60000)["default"]('sftpContinueOnProblems', false)["default"]('sftpFileWithTimestamp', false).demand(['projectKey']).argv;

logOptions = {
  name: "" + package_json.name + "-" + package_json.version,
  streams: [
    {
      level: 'error',
      stream: process.stderr
    }, {
      level: argv.logLevel,
      path: "" + argv.logDir + "/" + package_json.name + ".log"
    }
  ]
};

if (argv.logSilent) {
  logOptions.silent = argv.logSilent;
}

logger = new ExtendedLogger({
  additionalFields: {
    project_key: argv.projectKey
  },
  logConfig: logOptions
});

if (argv.logSilent) {
  logger.bunyanLogger.trace = function() {};
  logger.bunyanLogger.debug = function() {};
}

process.on('SIGUSR2', function() {
  return logger.reopenFileStreams();
});

process.on('exit', (function(_this) {
  return function() {
    return process.exit(_this.exitCode);
  };
})(this));

importFn = function(importer, fileName) {
  if (!fileName) {
    throw new Error('You must provide a file to be processed');
  }
  logger.debug("About to process file " + fileName);
  return fs.readFileAsync(fileName, {
    encoding: 'utf-8'
  }).then(function(content) {
    logger.debug('File read, running import');
    return importer.run(content);
  }).then(function() {
    return Promise.resolve(fileName);
  });
};

readJsonFromPath = function(path) {
  if (!path) {
    return Promise.resolve({});
  }
  return fs.readFileAsync(path, {
    encoding: 'utf-8'
  }).then(function(content) {
    return Promise.resolve(JSON.parse(content));
  });
};

ProjectCredentialsConfig.create().then((function(_this) {
  return function(credentials) {
    var file, options, orderStatusImport;
    options = {
      config: credentials.enrichCredentials({
        project_key: argv.projectKey,
        client_id: argv.clientId,
        client_secret: argv.clientSecret
      }),
      timeout: argv.timeout,
      user_agent: "" + package_json.name + " - " + package_json.version
    };
    if (argv.sphereHost) {
      options.host = argv.sphereHost;
    }
    orderStatusImport = new OrderStatusImport(logger, options);
    file = argv.file;
    if (file) {
      return importFn(orderStatusImport, file).then(function() {
        return _this.exitCode = 0;
      })["catch"](function(error) {
        logger.error(error, 'Oops, something went wrong when processing file (no SFTP)!');
        return _this.exitCode = 1;
      }).done();
    } else {
      tmp.setGracefulCleanup();
      return readJsonFromPath(argv.sftpCredentials).then(function(sftpCredentials) {
        var host, password, projectSftpCredentials, sftpHelper, username, _ref1;
        projectSftpCredentials = sftpCredentials[argv.projectKey] || {};
        _ref1 = _.defaults(projectSftpCredentials, {
          host: argv.sftpHost,
          username: argv.sftpUsername,
          password: argv.sftpPassword
        }), host = _ref1.host, username = _ref1.username, password = _ref1.password;
        if (!host) {
          throw new Error('Missing sftp host');
        }
        if (!username) {
          throw new Error('Missing sftp username');
        }
        if (!password) {
          throw new Error('Missing sftp password');
        }
        sftpHelper = new SftpHelper({
          host: host,
          username: username,
          password: password,
          sourceFolder: argv.sftpSource,
          targetFolder: argv.sftpTarget,
          fileRegex: argv.sftpFileRegex,
          logger: logger
        });
        return tmp.dirAsync({
          unsafeCleanup: true
        }).then(function(tmpPath) {
          logger.debug("Tmp folder created at " + tmpPath);
          return sftpHelper.download(tmpPath).then(function(files) {
            var filesSkipped, filesToProcess;
            logger.debug(files, "Processing " + files.length + " files...");
            filesToProcess = argv.sftpMaxFilesToProcess && _.isNumber(argv.sftpMaxFilesToProcess) && argv.sftpMaxFilesToProcess > 0 ? (logger.info("Processing max " + argv.sftpMaxFilesToProcess + " files"), _.first(files, argv.sftpMaxFilesToProcess)) : files;
            filesSkipped = 0;
            return Promise.map(filesToProcess, function(file) {
              return importFn(orderStatusImport, "" + tmpPath + "/" + file).then(function() {
                var filename, ts;
                if (argv.sftpFileWithTimestamp) {
                  ts = (new Date()).getTime();
                  filename = (function() {
                    switch (false) {
                      case !file.match(/\.xml$/i):
                        return file.substring(0, file.length - 4) + ("_" + ts + ".xml");
                      default:
                        return file;
                    }
                  })();
                } else {
                  filename = file;
                }
                logger.debug("Finishing processing file " + file);
                return sftpHelper.finish(file, filename).then(function() {
                  logger.info("File " + filename + " was successfully processed and marked as done on remote SFTP server");
                  return Promise.resolve();
                });
              })["catch"](function(err) {
                if (argv.sftpContinueOnProblems) {
                  filesSkipped++;
                  logger.warn(err, "There was an error processing the file " + file + ", skipping and continue");
                  return Promise.resolve();
                } else {
                  return Promise.reject(err);
                }
              });
            }, {
              concurrency: 1
            }).then((function(_this) {
              return function() {
                var totFiles;
                totFiles = _.size(filesToProcess);
                if (totFiles > 0) {
                  logger.info("Import successfully finished: " + (totFiles - filesSkipped) + " out of " + totFiles + " files were processed");
                } else {
                  logger.info("Import successfully finished: there were no new files to be processed");
                }
                return _this.exitCode = 0;
              };
            })(this));
          });
        })["catch"](function(error) {
          logger.error(error, 'Oops, something went wrong!');
          return _this.exitCode = 1;
        }).done();
      })["catch"](function(err) {
        logger.error(err, "Problems on getting sftp credentials from config files.");
        return _this.exitCode = 1;
      }).done();
    }
  };
})(this))["catch"]((function(_this) {
  return function(err) {
    logger.error(err, "Problems on getting client credentials from config files.");
    return _this.exitCode = 1;
  };
})(this)).done();
