/* ===========================================================
# sphere-orderstatus-xml-import - v0.1.0
# ==============================================================
# Copyright (c) 2013,2014 Sven Mueller
# Licensed under the MIT license.
*/
var Promise, Sftp, fs, fsExistsAsync, _;

_ = require('underscore');

Promise = require('bluebird');

fs = Promise.promisifyAll(require('fs'));

Sftp = require('sphere-node-utils').Sftp;

fsExistsAsync = function(path) {
  return new Promise(function(resolve, reject) {
    return fs.exists(path, function(exists) {
      if (exists) {
        return resolve(true);
      } else {
        return resolve(false);
      }
    });
  });
};

module.exports = (function() {

  /**
   * @constructor
   * Initialize {Sftp} client
   * @param {Object} [options] Configuration for {Sftp}
   */
  function _Class(options) {
    var host, password, username;
    if (options == null) {
      options = {};
    }
    host = options.host, username = options.username, password = options.password, this.sourceFolder = options.sourceFolder, this.targetFolder = options.targetFolder, this.fileRegex = options.fileRegex, this.logger = options.logger;
    this.sftpClient = new Sftp({
      host: host,
      username: username,
      password: password,
      logger: this.logger
    });
  }

  _Class.prototype.download = function(tmpFolder) {
    return fsExistsAsync(tmpFolder).then((function(_this) {
      return function(exists) {
        if (exists) {
          return Promise.resolve();
        } else {
          _this.logger.debug('Creating new tmp folder');
          return fs.mkdirAsync(tmpFolder);
        }
      };
    })(this)).then((function(_this) {
      return function() {
        return _this.sftpClient.openSftp();
      };
    })(this)).then((function(_this) {
      return function(sftp) {
        _this.logger.debug('New connection opened');
        _this._sftp = sftp;
        return _this.sftpClient.downloadAllFiles(sftp, tmpFolder, _this.sourceFolder, _this.fileRegex);
      };
    })(this)).then(function() {
      return fs.readdirAsync(tmpFolder);
    }).then(function(files) {
      return Promise.resolve(_.filter(files, function(fileName) {
        switch (false) {
          case !fileName.match(/\.xml$/i):
            return true;
          default:
            return false;
        }
      }));
    })["finally"]((function(_this) {
      return function() {
        _this.logger.debug('Closing connection');
        return _this.sftpClient.close(_this._sftp);
      };
    })(this));
  };

  _Class.prototype.finish = function(originalFileName, renamedFileName) {
    return this.sftpClient.openSftp().then((function(_this) {
      return function(sftp) {
        _this.logger.debug('New connection opened');
        _this._sftp = sftp;
        _this.logger.debug("Renaming file " + originalFileName + " to " + renamedFileName + " on the remote server");
        return _this.sftpClient.safeRenameFile(sftp, "" + _this.sourceFolder + "/" + originalFileName, "" + _this.targetFolder + "/" + renamedFileName);
      };
    })(this)).then(function() {
      return Promise.resolve();
    })["finally"]((function(_this) {
      return function() {
        _this.logger.debug('Closing connection');
        return _this.sftpClient.close(_this._sftp);
      };
    })(this));
  };

  return _Class;

})();
